program windir;

{$APPTYPE CONSOLE}

{
********************************************************
* windir 1.00 alpha                                    *
********************************************************
* by Sonic <mailto:sonic@pochtamt.ru>                  *
* GO TO [http://toysoft.narod.ru]                      *
*                                                      *
* DESCRIPTION:                                         *
*   Creates HTML-file with directories and files list, *
* needs  command-line  parameter  [path],  wich  means *
* what folder to scan.                                 *
********************************************************
}

uses
  SysUtils, Windows, Classes;

type
	TFileRec=class(TObject)
  	Name: string;
    Size: cardinal;
  end;
  TFolderRec=class;
  TFolderRec=class(TObject)
  	Name: string;
    Size: cardinal;
    Folders: TList;
    Files: TList;
    ParentFolder: TFolderRec;
    constructor Create;
    procedure Clear;
    procedure Sort;
    procedure CalcSize;
    procedure List(WriteAnchor: boolean=false);
    function FormSize(Size: cardinal): string;
  end;

const
	C_RD: string='.';  //root directory
	C_PD: string='..';  //parent directory
  C_HTML_FILE: string='list.htm'; //output HTML file name

var
  s: string;
  F: TFolderRec;
  HTML: TextFile;
  i: integer;

procedure Delim;
begin
  writeln('--------------------------------------------------------------------------------');
end;

procedure ExitProg(hc: integer);
begin
	Delim;
  write(#13'<Press ENTER to exit or close window yourself>');
  readln;
  Halt(hc);
end;

{ TFolderRec }

procedure TFolderRec.Clear;
var
	i: integer;
begin
	if Folders.Count>0 then begin
  	for i:=0 to Folders.Count-1 do begin
    	TFolderRec(Folders[i]).Clear;
    	TFolderRec(Folders[i]).Free;
    end;
  end;
  Folders.Clear;
	if Files.Count>0 then begin
  	for i:=0 to Files.Count-1 do begin
    	TFileRec(Files[i]).Free;
    end;
  end;
  Files.Clear;
end;

constructor TFolderRec.Create;
begin
	Folders:=TList.Create;
  Files:=TList.Create;
  ParentFolder:=nil;
end;

procedure FillFolder(Path: string; Folder: TFolderRec);
//fills TFolderRec object
// path - path to scanning folder WITHOUT '\'
// folder - TFolderRec object
var
  FF: THANDLE;
  FD: _WIN32_FIND_DATAA;
  F: TFolderRec;
  Fl: TFileRec;
  i: integer;
  vs: string;
begin
  //vs:=path+'\*';
  //CharToOem(PChar(vs),PChar(vs));
  //writeln('   (path is '''+vs+''')');
  if Folder=nil then begin
  	writeln('[ERROR #04] Internal error: object Folder not was created');
    ExitProg(4);
  end;
  FF:=FindFirstFile(PChar(path+'\*'),FD);
  if FF=INVALID_HANDLE_VALUE then begin
  	writeln('[ERROR #05] Searching error!');
    ExitProg(5);
  end;
  vs:=copy(Folder.Name,1,length(Folder.Name));
  CharToOem(PChar(vs),PChar(vs));
  writeln(' > searching in '''+vs+'''...');
  repeat
  	if FD.dwFileAttributes and FILE_ATTRIBUTE_DIRECTORY > 0 then begin
    	if (FD.cFileName<>C_RD) and (FD.cFileName<>C_PD) then begin
      	F:=TFolderRec.Create;
        F.Name:=FD.cFileName;
        F.ParentFolder:=Folder;
        Folder.Folders.Add(F);
			  vs:=copy(F.Name,1,length(F.Name));
			  CharToOem(PChar(vs),PChar(vs));
        writeln('   found folder '''+vs+'''');
      end;
    end else begin
    	Fl:=TFileRec.Create;
      Fl.Name:=FD.cFileName;
      Fl.Size:=(FD.nFileSizeHigh*MAXDWORD)+FD.nFileSizeLow;
      Folder.Files.Add(Fl);
		  vs:=copy(Fl.Name,1,length(Fl.Name));
		  CharToOem(PChar(vs),PChar(vs));
      writeln('   found file '''+vs+'''');
    end;
  until not FindNextFile(FF,FD);
  FindClose(FF);
  writeln(' > searching complete');
  if Folder.Folders.Count>0 then begin
	  vs:=copy(Folder.Name,1,length(Folder.Name));
  	CharToOem(PChar(vs),PChar(vs));
  	writeln(' > searching in subfolders of '''+
    	vs+'''...');
    for i:=0 to Folder.Folders.Count-1 do begin
	  	{writeln(' > searching in '''+
      	Path+'\'+TFolderRec(Folder.Folders[i]).Name+'''...');}
    	FillFolder(Path+'\'+TFolderRec(Folder.Folders[i]).Name,
      	TFolderRec(Folder.Folders[i]));
    end;
  	writeln(' > searching in subfolders of '''+
    	vs+''' complete');
  end;
end;

function CompareFNames(Item1, Item2: Pointer): Integer;
begin
	result:=AnsiCompareText(TFolderRec(Item1).Name,
  	TFolderRec(Item2).Name);
end;

function CompareFlNames(Item1, Item2: Pointer): Integer;
begin
	result:=AnsiCompareText(TFileRec(Item1).Name,
  	TFileRec(Item2).Name);
end;

procedure TFolderRec.Sort;
var
	i: integer;
begin
  Folders.Sort(@CompareFNames);
  Files.Sort(@CompareFlNames);
  if Folders.Count>0 then begin
  	for i:=0 to Folders.Count-1 do begin
    	TFolderRec(Folders[i]).Sort;
    end;
  end;
end;

procedure TFolderRec.CalcSize;
var
	i: integer;
begin
	Size:=0;
  if Files.Count>0 then begin
  	for i:=0 to Files.Count-1 do begin
    	inc(Size,TFileRec(Files[i]).Size);
    end;
  end;
  if Folders.Count>0 then begin
  	for i:=0 to Folders.Count-1 do begin
    	TFolderRec(Folders[i]).CalcSize;
      inc(Size,TFolderRec(Folders[i]).Size);
    end;
  end;
end;

procedure TFolderRec.List;
var
	i: integer;
  wa: boolean;
begin
	if (WriteAnchor) and (ParentFolder<>nil) then begin
  	write(HTML,'<a name="'+
    	IntToStr(ParentFolder.Folders.IndexOf(self))+'"></a>');
  end;
  write(HTML,'<ul>');
  if Folders.Count>0 then begin
  	for i:=0 to Folders.Count-1 do begin
    	write(HTML,'<li>'+TFolderRec(Folders[i]).Name+' '+
      	FormSize(TFolderRec(Folders[i]).Size));
      wa:=ParentFolder=nil;
      TFolderRec(Folders[i]).List(wa);
    end;
  end;
  if Files.Count>0 then begin
  	for i:=0 to Files.Count-1 do begin
    	write(HTML,'<li>'+TFileRec(Files[i]).Name+' '+
      	FormSize(TFolderRec(Files[i]).Size));
    end;
  end;
  write(HTML,'</ul>');
end;

function TFolderRec.FormSize;
var
	_Size: cardinal; 
begin
  _Size:=Size;
  {if _Size=0 then begin
    result:='0 ';
  	exit;
  end;}
  result:='<b>[<i>';
  if _Size<1024 then begin
		result:=result+IntToStr(_Size)+' </i>]</b>';
    exit;
  end;
  if _Size<1024*1024 then begin
		result:=result+Format('%f',[_Size/1024])+' </i>]</b>';
    exit;
  end;
  if _Size<1024*1024*1024 then begin
	 	result:=result+Format('%f',[_Size/(1024*1024)])+' </i>]</b>';
    exit;
  end;
 	result:=result+Format('%f',[_Size/(1024*1024*1024)])+' </i>]</b>';
end;

begin
  writeln('windir 1.00 alpha'#10#13);
  writeln('(C) TOY Soft [http://toysoft.narod.ru], 2004');
  writeln('(C) Sonic <sonic@pochtamt.ru>, 2004');
  writeln('EULA: freeware, "as is"');
  Delim;
  if ParamCount<1 then begin
    {MessageBeep(MB_ICONASTERISK);
  	MessageDlg(' (   )  !',
    	mtError,[mbOk],0);}
    writeln('[ERROR #01] Missing parameter - path to directory');
    ExitProg(1);
  end;
  s:=ParamStr(1);
  if not DirectoryExists(s) then begin
    writeln('[ERROR #02] Parameter - path to directory - is not valid');
    ExitProg(2);
  end;
  {if FileExists(s+'\'+C_HTML_FILE) then begin
  	writeln('[ERROR #03] File ''list.htm'' already exists');
    ExitProg(3);
  end;}
  writeln('>> reading tree...');
  F:=TFolderRec.Create;
  F.Name:=s;
  FillFolder(s,F);
  writeln('>> tree is read');
  if (F.Folders.Count=0) and (F.Files.Count=0) then begin
  	writeln(#10#13'[OK] Nothing to list, directory is empty');
    ExitProg(0);
  end;
  writeln('>> sorting...');
  F.Sort;
  writeln('>> sorting complete');
  writeln('>> calculating size...');
  F.CalcSize;
  writeln('>> calculation complete, total size is '+
  	IntToStr(F.Size)+' bytes');
  writeln('>> building HTML...');
  assignfile(HTML,s+'\'+C_HTML_FILE);
  rewrite(HTML);
  write(HTML,'<!DOCTYPE HTML PUBLIC '+
  	'"-//W3C//DTD HTML 4.0 Transitional//RU">'+
    '<html><head><title>windir 1.00 alpha -   '''+F.Name+
    '''</title><meta http-equiv='+
    '"Content-Type" content="text/html; '+
    'charset=windows-1251"><META NAME="Generator" '+
    'CONTENT="windir">'+
    '<style type="text/css">body {font-family: verdana;}'+
    '</style></head><body>');
  write(HTML,'<h1> </h1><hr> : '+
  	F.FormSize(F.Size));
  if F.Folders.Count>0 then begin
    write(HTML,{'<h2> :</h2>}'<h3>');
  	for i:=0 to F.Folders.Count-1 do begin
    	write(HTML,'<a href="#'+InttoStr(i)+'">'+
      	TFolderRec(F.Folders[i]).Name+'</a><br>');
    end;
    write(HTML,'</h3><hr>');
  end;
  F.List(true);
  write(HTML,'</body></html>');
  closefile(HTML);
  writeln('>> HTML is built');
  writeln('>> cleaning used memory...');
  F.Clear;
  F.Free;
  writeln(#10#13'[OK] Work coplete');
  ExitProg(0);
end.
